
FILENAMES = [".gitignore", "README.md", "get-pip.py", "buildout.cfg", "setup.in"]
FILENAMES_TO_UPDATE = ["bootstrap.py", "buildout.cfg", "setup.in"]

def get_files():
    from pkg_resources import resource_filename
    return [resource_filename(__name__, filename) for filename in FILENAMES]

def get_files_to_update():
    from pkg_resources import resource_filename
    return [resource_filename(__name__, filename) for filename in FILENAMES_TO_UPDATE]



import os
from infi.projector.plugins import CommandPlugin
from infi.projector.helper import assertions
from infi.projector.helper.utils import commit_changes_to_buildout, open_buildout_configfile
from infi.projector.helper.utils.package_sets import RepresentedListSet
from logging import getLogger

logger = getLogger(__name__)

USAGE = """
Usage:
    projector js-requirements list
    projector js-requirements add <requirement> [--commit-changes]
    projector js-requirements remove <requirement> [--commit-changes]
    projector js-requirements freeze [--commit-changes] [--push-changes]
    projector js-requirements unfreeze [--commit-changes] [--push-changes]


Options:
    js-requirement list               Show all js-requirement
    js-requirement add                add a package to the list of project js-requirement
    js-requirement remove             remove a package from project requirement list
    js-requirement freeze             Creates a js_versions section (based on .package-lock.json, telling buildout to use specific versions)
    js-requirement unfreeze           Deletes the js_versions section, if it exists
    <requirement>                     requirement to add/remove
"""


class JSRequirementsPlugin(CommandPlugin):
    DEFAULT_DIRECTORY = "parts/js/"

    def get_docopt_string(self):
        return USAGE

    def get_command_name(self):
        return 'js-requirements'

    def get_methods(self):
        return [self.list, self.add, self.remove, self.freeze, self.unfreeze]

    def get_package_set(self):
        return RepresentedListSet('js-requirements', 'javascript-packages')

    def list(self):
        from pprint import pprint
        pkg_set = self.get_package_set()
        if pkg_set.get():
            pprint(sorted(list(pkg_set.get()), key=lambda s: s.lower()))
        else:
            print('Please initiate js-requirements first by using "add" argument.')

    def remove(self):
        package_set = self.get_package_set()
        requirements = package_set.get()
        requirement = self.arguments.get('<requirement>')
        if requirement in requirements:
            requirements.remove(requirement)
            package_set.set(requirements)
        if self.arguments.get("--commit-changes", False):
            commit_message = "remove {} from js-requirements".format(requirement)
            commit_changes_to_buildout(commit_message)

    def add(self):
        with open_buildout_configfile(write_on_exit=True) as buildout_cfg:
            if not buildout_cfg.has_section("js-requirements"):
                buildout_cfg.add_section("js-requirements")
                buildout_cfg.set("js-requirements", "recipe", "infi.recipe.js_requirements")
                buildout_cfg.set("js-requirements", "js-directory", "")
                buildout_cfg.set("js-requirements", "symlink-to-directory", "parts/js")
                buildout_cfg.set("js-requirements", "javascript-packages", "[]")

        package_set = self.get_package_set()
        requirements = package_set.get()
        requirement = self.arguments.get('<requirement>')
        if requirement not in requirements:
            requirements.add(requirement)
            package_set.set(requirements)
        if self.arguments.get("--commit-changes", False):
            commit_message = "adding {} to js-requirements".format(requirement)
            commit_changes_to_buildout(commit_message)

    def freeze(self):
        from gitpy import LocalRepository
        from os import curdir
        import json

        # Read/write the buildout.cfg
        with open_buildout_configfile(write_on_exit=True) as buildout_cfg:
            if not buildout_cfg.has_section('js-requirements'):
                print("Missing js-requirements section")
                return
            packages_path = buildout_cfg.get('js-requirements', 'js-directory') or self.DEFAULT_DIRECTORY
            try:
                with open(os.path.join(packages_path, '.package-lock.json'), 'r') as pljson:
                    selected_versions = json.load(pljson)
                    if buildout_cfg.has_section("js_versions"):
                        buildout_cfg.remove_section("js_versions")
                    buildout_cfg.add_section("js_versions")
                for key in sorted(selected_versions.keys(), key=lambda s: s.lower()):
                    buildout_cfg.set("js_versions", key, selected_versions[key])
                buildout_cfg.set('buildout', 'js_versions', True)
            except IOError as e:
                import errno
                print(e.message)
                if e.errno == errno.ENOENT:
                    print('.package-lock.json file is missing, try running projector devenv build to create the file')

        # Git operations
        repository = LocalRepository(curdir)
        if self.arguments.get("--commit-changes", False):
            repository.add("buildout.cfg")
            repository.commit("Freezing javascript dependencies")
        push_changes = self.arguments.get("--push-changes", False)
        if push_changes:
            repository._executeGitCommandAssertSuccess("git push")

    def unfreeze(self):
        from gitpy import LocalRepository
        from os import curdir
        with open_buildout_configfile(write_on_exit=True) as buildout_cfg:
            if not buildout_cfg.has_section('js-requirements'):
                print("Missing js-requirements section")
                return
            buildout_cfg.remove_option("buildout", "js_versions")
            buildout_cfg.remove_section("js_versions")

        # Git operations
        repository = LocalRepository(curdir)
        if self.arguments.get("--commit-changes", False):
            repository.add("buildout.cfg")
            repository.commit("Unfreezing javascript dependencies")
        push_changes = self.arguments.get("--push-changes", False)
        if push_changes:
            repository._executeGitCommandAssertSuccess("git push")



from infi.projector.plugins import CommandPlugin
from infi.projector.helper import assertions
from infi.projector.helper.utils import commit_changes_to_buildout, commit_changes_to_manifest_in
from infi.projector.helper.utils.package_sets import PackageDataSet
from logging import getLogger

logger = getLogger(__name__)

USAGE = """
Usage:
    projector package-data list
    projector package-data add <filename> [--commit-changes]
    projector package-data remove <filename> [--commit-changes]

Options:
    <filename>              file to add
"""

class PackageDataPlugin(CommandPlugin):
    def get_docopt_string(self):
        return USAGE

    def get_command_name(self):
        return 'package-data'

    def get_methods(self):
        return [self.list, self.add, self.remove]

    @assertions.requires_repository
    def pre_command_assertions(self):
        pass

    def get_package_set(self):
        return PackageDataSet()

    def list(self):
        from pprint import pprint
        pprint(sorted(list(self.get_package_set().get()), key=lambda s: s.lower()))

    def write_manifest_in(self, data_set):
        data_list = sorted(list(data_set), key=lambda s: s.lower())
        with open("MANIFEST.in", 'w') as fd:
            fd.write("recursive-include src {}\n".format(' '.join(data_list)))

    def remove(self):
        package_set = self.get_package_set()
        data_set = package_set.get()
        filename = self.arguments.get('<filename>')
        if filename in data_set:
            data_set.remove(filename)
            package_set.set(data_set)
        self.write_manifest_in(data_set)
        if self.arguments.get("--commit-changes", False):
            commit_message = "removing {} from package data".format(filename)
            commit_changes_to_buildout(commit_message)
            commit_changes_to_manifest_in(commit_message)

    def add(self):
        package_set = self.get_package_set()
        data_set = package_set.get()
        filename = self.arguments.get('<filename>')
        if filename not in data_set:
            data_set.add(filename)
            package_set.set(data_set)
        self.write_manifest_in(data_set)
        if self.arguments.get("--commit-changes", False):
            commit_message = "adding {} to package data".format(filename)
            commit_changes_to_buildout(commit_message)
            commit_changes_to_manifest_in(commit_message)



__version__ = "1.0.6"
__git_commiter_name__ = "Arnon Yaari"
__git_commiter_email__ = "arnony@infinidat.com"
__git_branch__ = u'(HEAD detached at v1.0.6)'
__git_remote_tracking_branch__ = '(No remote tracking)'
__git_remote_url__ = '(Not remote tracking)'
__git_head_hash__ = '92123534be92829f17527ff0a45984eacadf8cce'
__git_head_subject__ = u'Finished Release v1.0.6'
__git_head_message__ = u''
__git_dirty_diff__ = u''
__git_commit_date__ = '2018-08-21 14:20:41'



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Bootstrap a buildout-based project

Simply run this script in a directory containing a buildout.cfg.
The script accepts buildout command-line options, so you can
use the -c option to specify an alternate configuration file.
"""

import os
import shutil
import sys
import tempfile
import glob
import re

from optparse import OptionParser

tmpeggs = tempfile.mkdtemp()

usage = '''\
[DESIRED PYTHON FOR BUILDOUT] bootstrap.py [options]

Bootstraps a buildout-based project.

Simply run this script in a directory containing a buildout.cfg, using the
Python that you want bin/buildout to use.

Note that by using --find-links to point to local resources, you can keep
this script from going over the network.
'''

def normalize_to_url(option, opt_str, value, parser):
    # copied from zc.buildout-1.x bootstrap.py
    try:
        from urllib.request import pathname2url
    except ImportError:
        from urllib import pathname2url
    if value:
        if '://' not in value:  # It doesn't smell like a URL.
            value = 'file://%s' % (
                pathname2url(
                os.path.abspath(os.path.expanduser(value))),)
        if opt_str == '--download-base' and not value.endswith('/'):
            # Download base needs a trailing slash to make the world happy.
            value += '/'
    else:
        value = None
    name = opt_str[2:].replace('-', '_')
    setattr(parser.values, name, value)

ezsetup_source = 'https://bootstrap.pypa.io/ez_setup.py'
setuptools_source = "https://pypi.io/packages/source/s/setuptools/"
pypi_index = "https://pypi.python.org/simple/"

parser = OptionParser(usage=usage)
parser.add_option("-v", "--version", help="use a specific zc.buildout version")

parser.add_option("-t", "--accept-buildout-test-releases",
                  dest='accept_buildout_test_releases',
                  action="store_true", default=False,
                  help=("Normally, if you do not specify a --version, the "
                        "bootstrap script and buildout gets the newest "
                        "*final* versions of zc.buildout and its recipes and "
                        "extensions for you.  If you use this flag, "
                        "bootstrap and buildout will get the newest releases "
                        "even if they are alphas or betas."))
parser.add_option("-c", "--config-file",
                  help=("Specify the path to the buildout configuration "
                        "file to be used."))
parser.add_option("-f", "--find-links",
                  help=("Specify a URL to search for buildout releases"))
# in closed networks, one needs to provide the location of ez_setup.py
parser.add_option("--setup-source", action="callback", dest="setup_source",
                  callback=normalize_to_url, nargs=1, type="string",
                  help=("Specify a URL or file location for the setuptool's ez_setup.py"),
                  default=ezsetup_source)
# in closed networks, one needs to provide the location of the setuptools and buildout archives
parser.add_option("--download-base", action="callback", dest="download_base",
                  callback=normalize_to_url, nargs=1, type="string",
                  help=("Specify a URL or directory for downloading setuptools and buildout"),
                  default=setuptools_source)
# in closed networks, one needs to provide an index server to find buildout on
parser.add_option("--index-url", action="callback", dest="index_url",
                  callback=normalize_to_url, nargs=1, type="string",
                  help=("Specify an alternative for PyPI simple index url"),
                  default=pypi_index)
# explicit setuptools version
parser.add_option("--setuptools-version", action="store", dest="setuptools_version", type="string",
                  help=("Specifiy a specific version of setuptools to use"),
                  )
options, args = parser.parse_args()

######################################################################
# load/install setuptools

def _cleanup_old_zc_buildout_modules():
    # installing setuptools imported the site module, which added all the stuff in site-packages to sys.path,
    # even though in the case Python was executed -S.
    # we want to remove all traces for this
    for item in sys.path:
        if item.endswith('zc') or item.endswith('buildout'):
            sys.path.remove(item)
        if os.path.exists(os.path.join(item, 'zc')) and 'site-packages' in item:
            sys.path.remove(item)
    for module in sys.modules.keys():
        if 'zc' in module:
            del sys.modules[module]

def _cleanup_setuptools_and_distribute_modules():
    # installing setuptools imported the site module, which added all the stuff in site-packages to sys.path,
    # even though in the case Python was executed -S.
    # we want to remove all traces for this
    paths_to_remove = [item for item in sys.path if 'setuptools-' in item or 'distribute-' in item]
    # the python-setuptools installs setuptools in a different way
    paths_to_remove.extend(item for item in sys.path if glob.glob(os.path.join(item, 'setuptools-*'))
                           and 'dist-packages' in item)
    paths_to_remove.extend(item for item in sys.path if glob.glob(os.path.join(item, 'distribute-*'))
                           and 'dist-packages' in item)
    # virtualenv creates under site-packages a directory named setuptools/ with an __init__.py inside in
    # this causes makes setuptools importable if that site-packages is in sys.path
    # in this case, pkg_resources is directly under site-packages/
    # so it must go
    paths_to_remove.extend(item for item in sys.path if
                           (os.path.exists(os.path.join(item, "setuptools")) or
                           os.path.exists(os.path.join(item, "pkg_resources")) or
                           os.path.exists(os.path.join(item, "pkg_resources.py"))) and
                           item.endswith('site-packages'))
    sys.path = list(set(sys.path) - set(paths_to_remove))


to_reload = False
try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse

try:
    from urllib.request import urlopen
except ImportError:
    from urllib2 import urlopen

_cleanup_setuptools_and_distribute_modules()
_cleanup_setuptools_and_distribute_modules()  # wtf need to run twice

try:
    import pkg_resources
    import setuptools
except ImportError:
    ez = {}

    # ez_setup tries to use curl/wget where available, we want to avoid ssl issues
    # so we want to modify it to ignore them
    # download_file_powershell.viable = has_powershell
    # download_file_curl.viable = has_curl
    # download_file_wget.viable = has_wget
    # all of these need to change
    ez_setup_source = urlopen(options.setup_source).read()
    ez_setup_source_fixed = ez_setup_source.replace(b'.viable = h', b'.viable = lambda: False #')
    exec(ez_setup_source_fixed, ez)
    # wheh running python -S bootstap.py, when setuptools is in site-packages
    # but not included in sys.path due to the -S flag,
    # exec() somehows puts all the stuff from site-packages in sys.path
    # this can cause a version conflict for setuptools, when we want version X,
    # but version Y is in site-packages
    # this is a workaround for this issue
    _cleanup_setuptools_and_distribute_modules()
    setup_args = dict(to_dir=tmpeggs, download_delay=0)
    if options.download_base:
        setup_args['download_base'] = options.download_base
        if options.download_base.startswith("file://"):
            download_base = urlparse(options.download_base).path
            if os.path.exists(download_base):
                files = glob.glob(os.path.join(download_base, "setuptools-*.tar.gz"))
                if len(files) == 1:
                    setuptools_version = re.match("setuptools-(?P<version>.*).tar.gz",
                                                  os.path.basename(files[0])).groupdict()['version']
                    setup_args['version'] = setuptools_version
    if options.setuptools_version:
        setup_args['version'] = options.setuptools_version
    # workaround for telling setuptools to use urlopen
    ez['get_best_downloader'] = lambda: ez['download_file_insecure']
    ez['use_setuptools'](**setup_args)

    if to_reload:
        reload(pkg_resources)
    import pkg_resources
    # This does not (always?) update the default working set.  We will
    # do it.
    for path in sys.path:
        if path not in pkg_resources.working_set.entries:
            pkg_resources.working_set.add_entry(path)

######################################################################
# Install buildout

ws = pkg_resources.working_set

cmd = [sys.executable, '-c',
       'from setuptools.command.easy_install import main; main()',
       '-mZqNxd', tmpeggs]

find_links = os.environ.get(
    'bootstrap-testing-find-links',
    options.find_links or
    ('http://downloads.buildout.org/'
     if options.accept_buildout_test_releases else None)
    )
if find_links:
    cmd.extend(['-f', find_links])
if options.index_url:
    cmd.extend(["-i", options.index_url])
setuptools_path = ws.find(
    pkg_resources.Requirement.parse('setuptools')).location

requirement = 'zc.buildout'
version = options.version
if version is None and not options.accept_buildout_test_releases:
    # Figure out the most recent final version of zc.buildout.
    import setuptools.package_index
    _final_parts = '*final-', '*final'

    def _final_version(parsed_version):
        for part in parsed_version:
            if (part[:1] == '*') and (part not in _final_parts):
                return False
        return True
    kwargs = dict(search_path=[setuptools_path])
    if options.index_url:
        kwargs['index_url'] = options.index_url
    index = setuptools.package_index.PackageIndex(**kwargs)
    if find_links:
        index.add_find_links((find_links,))
    if options.download_base:
        index.add_find_links((options.download_base,))
    req = pkg_resources.Requirement.parse(requirement)
    if index.obtain(req) is not None:
        best = []
        bestv = None
        for dist in index[req.project_name]:
            distv = dist.parsed_version
            if _final_version(distv):
                if bestv is None or distv > bestv:
                    best = [dist]
                    bestv = distv
                elif distv == bestv:
                    best.append(dist)
        if best:
            best.sort()
            version = best[-1].version
if version:
    requirement = '=='.join((requirement, version))
cmd.append(requirement)

import subprocess
if subprocess.call(cmd, env=dict(os.environ, PYTHONPATH=setuptools_path)) != 0:
    raise Exception(
        "Failed to execute command:\n%s",
        repr(cmd)[1:-1])

######################################################################
# Import and run buildout
# installing setuptools imported site.py, which added zc.buildout to the WorkingSet if it was previously installed
# this may raise a VerionConflict here; we just need to resolve the location of the buildout we just installed
# so we clear the WorkingSet
_cleanup_old_zc_buildout_modules()
ws.by_key = {}
ws.add_entry(setuptools_path)
ws.add_entry(tmpeggs)
ws.require(requirement)
import zc.buildout.buildout

if not [a for a in args if '=' not in a]:
    args.append('bootstrap')

# if -c was provided, we push it back into args for buildout' main function
if options.config_file is not None:
    args[0:0] = ['-c', options.config_file]

zc.buildout.buildout.main(args)
shutil.rmtree(tmpeggs)



from contextlib import contextmanager
from infi.projector.plugins import CommandPlugin
from infi.projector.helper import assertions, utils
from infi.projector.helper.utils import configparser
from logging import getLogger

logger = getLogger(__name__)

USAGE = """
Usage:
    projector devenv build [--clean] [--force-bootstrap] [--no-submodules] [--no-setup-py] [--no-js-requirements] [--no-scripts] [--use-isolated-python] [[--newest] | [--offline] | [--prefer-final]]
    projector devenv relocate ([--absolute] | [--relative]) [--commit-changes]
    projector devenv pack

Options:
    devenv build            use this command to generate setup.py and the console scripts
    devenv relocate         use this command to switch from relative and absolute paths in the console scripts
    devenv pack             create a package, e.g. deb/rpm/msi
    --clean                 clean build-related files and directories before building
    --force-bootstrap       run buildout bootstrap even if the buildout script already exists
    --no-submodules         do not clone git sub-modules defined in buildout.cfg
    --no-scripts            do not install the dependent packages, nor create the console scripts. just create setup.py
    --use-isolated-python   do not use global system python in console scripts, use Infinidat's isolated python builds
    --newest                always check for new package version on PyPI
    --offline               install packages only from download cache (no internet connection)
    --absolute              change the paths in the development environment to absolute paths
    --relative              change the paths in the development environment to relative paths
    --prefer-final          don't install development versions of dependencies, prefer their latest final versions.
    --no-js-requirements    don't download and extract js-requirements.
"""


class DevEnvPlugin(CommandPlugin):
    def get_docopt_string(self):
        return USAGE

    def get_command_name(self):
        return 'devenv'

    def get_methods(self):
        return [self.build, self.relocate, self.pack]

    @assertions.requires_repository
    def pre_command_assertions(self):
        pass

    def create_cache_directories(self):
        from os import makedirs
        from os.path import join, exists
        with utils.open_buildout_configfile() as buildout:
            cachedir = buildout.get("buildout", "download-cache")
        cache_dist = join(cachedir, "dist")
        if not exists(cache_dist):
            makedirs(cache_dist)

    def _get_pypi_index_url(self):
        from os import path
        pydistutils = configparser.ConfigParser()
        pydistutils.read([path.expanduser(path.join("~", basename)) for basename in ['.pydistutils.cfg', 'pydistutils.cfg']])
        try:
            return pydistutils.get("easy_install", "index-url").strip("/")
        except (configparser.NoSectionError, configparser.NoOptionError):
            return "https://pypi.python.org/simple"

    def bootstrap_if_necessary(self):
        from os.path import join, split
        from os import name
        from sys import argv
        from pkg_resources import resource_filename
        from infi.projector.plugins.builtins.repository import skeleton
        buildout_executable_exists = assertions.is_executable_exists(join("bin", "buildout"))
        if not buildout_executable_exists or self.arguments.get("--force-bootstrap", False) or self.arguments.get("--newest", False):
            try:
                return utils.execute_assert_success([utils.get_executable('buildout'), 'bootstrap'])
            except OSError:  # workaround for OSX
                pass
            try:
                utils.execute_assert_success(['buildout', 'bootstrap'])
            except OSError:
                dirname, basename = split(argv[0])
                buildout = join(dirname, 'buildout.exe' if name == 'nt' else 'buildout')
                utils.execute_assert_success([buildout, 'bootstrap'])

    def install_sections_by_recipe(self, recipe, stripped=True):
        with utils.open_buildout_configfile() as buildout:
            sections_to_install = [section for section in buildout.sections()
                                   if buildout.has_option(section, "recipe") and
                                      buildout.get(section, "recipe").startswith(recipe)]
        if sections_to_install:
            utils.execute_with_buildout("install {}".format(' '.join(sections_to_install)), stripped=stripped)

    def submodule_update(self):
        with utils.buildout_parameters_context(['buildout:develop=']):
            self.install_sections_by_recipe("zerokspot.recipe.git")
            self.install_sections_by_recipe("gitrecipe")
            self.install_sections_by_recipe("git-recipe")

    def download_js_requirements(self):
        with utils.buildout_parameters_context(['buildout:develop=']):
            self.install_sections_by_recipe('infi.recipe.js_requirements')

    def create_setup_py(self):
        with utils.buildout_parameters_context(['buildout:develop=']):
            self.install_sections_by_recipe("infi.recipe.template.version")

    def get_isolated_python_section_name(self):
        with utils.open_buildout_configfile() as buildout:
            sections = [section for section in buildout.sections()
                        if buildout.has_option(section, "recipe") and
                        buildout.get(section, "recipe").startswith("infi.recipe.python")
                        and not buildout.get(section, "recipe").endswith(":pack")]
        return sections[0]

    def create_scripts(self):
        additional_options = ["buildout:prefer-final=true"] if self.arguments.get("--prefer-final") else []
        with utils.buildout_parameters_context(additional_options):
            self.install_sections_by_recipe("infi.recipe.console_scripts")

    def _remove_files_of_type_recursively(self, root_path, file_type):
        import os
        file_type = file_type if file_type.startswith(".") else "." + file_type
        [[os.remove(os.path.join(p, f)) for f in fs if f.endswith(file_type)] for p, ds, fs in os.walk(root_path)]

    def clean_build(self):
        from os.path import exists
        from os import remove
        from shutil import rmtree
        directories_to_clean = ['bin', 'eggs', 'develop-eggs']
        files_to_clean = ['setup.py']
        [remove(filename) for filename in files_to_clean if exists(filename)]
        [rmtree(dirname) for dirname in directories_to_clean if exists(dirname)]
        self._remove_files_of_type_recursively("src", "pyc")

    @contextmanager
    def buildout_newest_or_offline_context(self):
        parameters = []
        if self.arguments.get('--newest'):
            parameters.append('-n')
        if self.arguments.get('--offline'):
            parameters.append('-o')
        with utils.buildout_parameters_context(parameters):
            yield

    def _remove_setuptools_egg_link(self):
        # HOSTDEV-1130
        # https://bugs.launchpad.net/zc.buildout/+bug/1210996
        import os
        with utils.open_buildout_configfile() as buildout:
            try:
                develop_eggs_dir = buildout.get("buildout", "develop-eggs-directory")
            except (configparser.NoSectionError, configparser.NoOptionError):
                develop_eggs_dir = "develop-eggs"
            setuptools_egg_link = os.path.join(develop_eggs_dir, "setuptools.egg-link")
            if os.path.exists(setuptools_egg_link):
                os.remove(setuptools_egg_link)

    def _get_pip(self):
        from pkg_resources import resource_filename
        from infi.projector.plugins.builtins.repository import skeleton
        get_pip_py = resource_filename(skeleton.__name__, "get-pip.py")
        with open("get-pip.py", "w") as dst:
            with open(get_pip_py) as src:
                dst.write(src.read())

    def _install_setuptools_and_zc_buildout(self):
        from os.path import join, exists
        from os import environ, remove

        with utils.open_buildout_configfile() as buildout:
            cachedir = buildout.get("buildout", "download-cache")
        cache_dist = join(cachedir, "dist")

        cmd = []
        packages = []

        # in case dependencies are frozen, we need to use the frozen version of setuptools and zc.buildout
        with utils.open_buildout_configfile() as buildout:
            for package in ['setuptools', 'zc.buildout', 'pip']:
                if buildout.has_option("versions", package):
                    packages += ['{}=={}'.format(package, buildout.get("versions", package))]
                else:
                    packages += [package]

        env = environ.copy()
        env['PYTHONPATH'] = ''
        utils.execute_assert_success([utils.get_isolated_executable('python'), 'get-pip.py', '--prefix=%s' % join('parts', 'python')] + packages, env=env)
        remove('get-pip.py')
        utils.execute_assert_success([utils.get_isolated_executable('python'), '-m', 'pip', 'download', '--dest', cache_dist] + packages, env=env)

    def install_isolated_python_if_necessary(self):
        from os import environ
        if not self.arguments.get("--use-isolated-python", False):
            return
        self._remove_setuptools_egg_link()
        if not assertions.is_isolated_python_exists() or self.arguments.get("--newest", False):
            with utils.buildout_parameters_context(['buildout:develop=', 'buildout:versions=no', 'no:key=value']):
                utils.execute_with_buildout("install {}".format(self.get_isolated_python_section_name()))
        self._get_pip()
        self._install_setuptools_and_zc_buildout()
        env = environ.copy()
        env['PYTHONPATH'] = ''
        utils.execute_assert_success([utils.get_isolated_executable('buildout'), 'bootstrap'], env=env)

    def build(self):
        if self.arguments.get("--clean", False):
            self.clean_build()
        elif self.arguments.get("--newest", False):
            self._remove_files_of_type_recursively("src", "pyc")
        self.create_cache_directories()
        self.bootstrap_if_necessary()
        with self.buildout_newest_or_offline_context():
            self.install_isolated_python_if_necessary()
            if not self.arguments.get("--no-submodules", False):
                self.submodule_update()
            if not self.arguments.get("--no-setup-py", False):
                self.create_setup_py()
            if not self.arguments.get("--no-scripts", False):
                self.create_scripts()
            if not self.arguments.get("--no-js-requirements", False):
                self.download_js_requirements()

    def relocate(self):
        relative_paths = self.arguments.get("--relative", False)
        with utils.open_buildout_configfile(write_on_exit=True) as buildout:
            buildout.set("buildout", "relative-paths", 'true' if relative_paths else 'false')
        if self.arguments.get("--commit-changes", False):
            commit_message = "Changing shebang to {} paths".format("relative" if relative_paths else "absolute")
            utils.commit_changes_to_buildout(commit_message)
        logger.info("Configuration changed. Run `projector devenv build [--use-isolated-python]`.")

    def pack(self):
        assertions.assert_isolated_python_exists()

        self.install_sections_by_recipe("infi.recipe.application_packager", stripped=False)



from infi.projector.plugins import CommandPlugin
from infi.projector.helper import assertions
from infi.projector.helper.utils import open_buildout_configfile, commit_changes_to_buildout
from logging import getLogger

logger = getLogger(__name__)

USAGE = """
Usage:
    projector package-scripts show (--post-install | --pre-uninstall)
    projector package-scripts set <value> (--post-install | --pre-uninstall) [--commit-changes]

Options:
    <value>                 the exectuable basename, under the 'bin' directory (e.g. projector). Set None to disable.
    --post-install          perform the action for the post install script
    --pre-uninstall         perform the action for the pre uninstall script
"""

class PackageScriptsPlugin(CommandPlugin):
    def get_docopt_string(self):
        return USAGE

    def get_command_name(self):
        return 'package-scripts'

    def get_methods(self):
        return [self.show, self.set]

    @assertions.requires_repository
    def pre_command_assertions(self):
        pass

    def get_attribute(self):
        if self.arguments.get('--post-install'):
            return "post_install_script_name"
        if self.arguments.get('--pre-uninstall'):
            return "pre_uninstall_script_name"

    def get_value(self):
        with open_buildout_configfile() as buildout_cfg:
            return buildout_cfg.get("project", self.get_attribute())

    def set_value(self, value):
        with open_buildout_configfile(write_on_exit=True) as buildout_cfg:
            return buildout_cfg.set("project", self.get_attribute(), value)

    def show(self):
        from pprint import pprint
        pprint(self.get_value())

    def set(self):
        value = self.arguments.get('<value>')
        self.set_value(value)
        if self.arguments.get("--commit-changes", False):
            commit_message = "setting {} to {}".format(self.get_attribute(), value)
            commit_changes_to_buildout(commit_message)




SETUP_INFO = dict(
    name = 'infi.projector',
    version = '1.0.6',
    author = 'Arnon Yaari',
    author_email = 'arnony@infinidat.com',

    url = 'https://github.com/Infinidat/infi.projector',
    license = 'BSD',
    description = """Python project management tool""",
    long_description = """For the complete document, see the README.md file over at GitHub""",

    # http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers = [
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],

    install_requires = [
'docopt',
'git-py',
'infi.execute',
'infi.recipe.python',
'infi.recipe.template.version',
'pip',
'setuptools',
'six',
'wheel',
'zc.buildout>=2.9.2'
],
    namespace_packages = ['infi'],

    package_dir = {'': 'src'},
    package_data = {'': [
'.gitignore',
'buildout.cfg',
'get-pip.py',
'README.md',
'setup.in'
]},
    include_package_data = True,
    zip_safe = False,

    entry_points = dict(
        console_scripts = ['projector = infi.projector.scripts:projector'],
        gui_scripts = [],
        projector_command_plugins = ['repository = infi.projector.plugins.builtins.repository:RepositoryPlugin', 'envenv = infi.projector.plugins.builtins.devenv:DevEnvPlugin', 'version = infi.projector.plugins.builtins.version:VersionPlugin', 'requirements = infi.projector.plugins.builtins.requirements:RequirementsPlugin', 'console_scripts = infi.projector.plugins.builtins.console_scripts:ConsoleScriptsPlugin', 'gui_scripts = infi.projector.plugins.builtins.gui_scripts:GuiScriptsPlugin', 'package_scripts = infi.projector.plugins.builtins.package_scripts:PackageScriptsPlugin', 'package_data = infi.projector.plugins.builtins.package_data:PackageDataPlugin', 'isolated_pyton = infi.projector.plugins.builtins.isolated_python:IsolatedPythonPlugin', 'submodules = infi.projector.plugins.builtins.submodules:SubmodulePlugin', 'js_requirements = infi.projector.plugins.builtins.js_requirements:JSRequirementsPlugin']),
    )

def setup():
    from setuptools import setup as _setup
    from setuptools import find_packages
    SETUP_INFO['packages'] = find_packages('src')
    _setup(**SETUP_INFO)

if __name__ == '__main__':
    setup()




from contextlib import contextmanager
from logging import getLogger
import sys
import re
try:
    import configparser
except ImportError:     # Python 2
    import ConfigParser as configparser

logger = getLogger(__name__)

BUILDOUT_PARAMETERS = []

class PrettyExecutionError(Exception):
    # infi.execute.ExecutionError does print stdout and stderr well, and this is a must when running buildout
    def __init__(self, result):
        encoding = getattr(sys.stdout, 'encoding', None) or 'utf-8'
        stdout = result.get_stdout()
        if stdout is not None:
            stdout = stdout.decode(encoding)
        stderr = result.get_stderr()
        if stderr is not None:
            stderr = stderr.decode(encoding)
        msg = "Execution of %r failed!\nresult=%s\nstdout=%s\nstderr=%s"
        msg = msg % (result._command, result.get_returncode(), stdout, stderr)
        super(PrettyExecutionError, self).__init__(msg)
        self.result = result

def _chdir_and_log(path):
    from os import chdir
    chdir(path)
    logger.debug("Changed directory to {!r}".format(path))

@contextmanager
def chdir(path):
    from os.path import abspath
    from os import curdir
    path = abspath(path)
    current_dir = abspath(curdir)
    _chdir_and_log(path)
    try:
        yield
    finally:
        _chdir_and_log(current_dir)

@contextmanager
def open_buildout_configfile(filepath="buildout.cfg", write_on_exit=False):
    parser = configparser.ConfigParser()
    parser.optionxform = str    # make options case-sensitive
    parser.read(filepath)
    try:
        yield parser
    finally:
        if not write_on_exit:
            return
        with open(filepath, 'w') as fd:
            parser.write(fd)

def is_running_inside_virtualenv():
    import sys
    from six import string_types
    return isinstance(getattr(sys, 'real_prefix', None), string_types)

def parse_args(commandline_or_args):
    return commandline_or_args if isinstance(commandline_or_args, list) else commandline_or_args.split()

def execute_assert_success(args, env=None):
    from infi import execute
    logger.info("Executing {}".format(' '.join(args)))
    result = execute.execute(args, env=env)
    if result.get_returncode() is not None and result.get_returncode() != 0:
        logger.error(result.get_stderr())
        raise PrettyExecutionError(result)

def _get_executable_from_shebang_line():  # pragma: no cover
    # The executable wrapper in distribute dynamically loads Python's DLL, which causes sys.executable to be the wrapper
    # and not the original python exeuctable. We have to find the real executable as Distribute does.
    from os import path
    import sys
    executable_script_py = sys.executable.replace(".exe", "-script.py")
    if not path.exists(executable_script_py):
        # using original Python executable
        return sys.executable
    with open(executable_script_py) as fd:
        shebang_line = fd.readlines()[0].strip()
    executable_path = path.normpath(shebang_line[2:])
    return (executable_path + '.exe') if not executable_path.endswith('.exe') else executable_path


def get_python_interpreter():
    import os
    import sys
    from ..assertions import is_windows
    if is_windows():
        return _get_executable_from_shebang_line()
    else:
        return os.path.join(sys.real_prefix, 'bin', 'python') if is_running_inside_virtualenv() else sys.executable


def get_executable(filename):
    import os
    dirpath, basename = os.path.split(get_python_interpreter())
    isolated_python_bin = os.path.join('parts', 'python', 'bin')
    filename_with_ext = (filename + '.exe') if basename.endswith('.exe') else filename
    # if we are under a buildout project, the scripts won't be inside the isolated python
    bin_dir = dirpath.replace(isolated_python_bin, 'bin') if \
              dirpath.endswith(isolated_python_bin) else dirpath
    if os.name == 'nt' and 'python' not in filename:
        # if we are write_on_exit windows, the console scripts are under Scripts
        for base_dir in [bin_dir, os.path.abspath(os.path.join(bin_dir, os.pardir))]:
            if os.path.exists(os.path.join(base_dir, 'Scripts')):
                bin_dir = os.path.join(base_dir, 'Scripts')
    return os.path.join(bin_dir, filename_with_ext)


def execute_with_python(commandline_or_args):
    import os
    import sys
    from ..assertions import is_windows
    args = parse_args(commandline_or_args)
    executable = [get_python_interpreter()]
    if not is_running_inside_virtualenv():
        executable.append('-S')
    try:
        execute_assert_success(executable + args)
    except PrettyExecutionError:
        if '-S' not in executable:
            raise
        logger.warning("Command failed with -S, trying without")
        executable.remove('-S')
        execute_assert_success(executable + args)


def get_isolated_executable(filename):
    import os
    from ..assertions import is_windows
    return os.path.join('parts', 'python',
                        'Scripts' if is_windows() and 'python' not in filename else 'bin',
                        '{}{}'.format(filename, '.exe' if is_windows() else ''))


def execute_with_isolated_python(commandline_or_args):
    import os
    from ..assertions import is_windows
    args = parse_args(commandline_or_args)
    executable = [get_isolated_executable('python')]
    with open_buildout_configfile() as buildout:
        if buildout.get('buildout', 'relative-paths') in ['True', 'true']:
            [executable] = os.path.abspath(executable[0])
    execute_assert_success(executable + args)

def execute_with_buildout(commandline_or_args, env=None, stripped=True):
    from os import name, path, environ
    _env = environ.copy()
    if env:
        _env.update(env)
    args = parse_args(commandline_or_args)
    python = path.join('bin', 'python{}'.format('.exe' if name == 'nt' else ''))
    buildout = path.join('bin', 'buildout{}'.format('.exe' if name == 'nt' else ''))
    buildout_script = path.join('bin', 'buildout{}'.format('-script.py' if name == 'nt' else ''))
    if path.exists(python) and not stripped:
        execute_assert_success([python, buildout_script] + BUILDOUT_PARAMETERS + args, env=_env)
    else:
        execute_assert_success([buildout] + BUILDOUT_PARAMETERS + args, env=_env)

@contextmanager
def buildout_parameters_context(parameters):
    try:
        [BUILDOUT_PARAMETERS.append(param) for param in parameters if param not in BUILDOUT_PARAMETERS]
        yield
    finally:
        [BUILDOUT_PARAMETERS.remove(param) for param in parameters if param in BUILDOUT_PARAMETERS]

def _release_version_in_git(version_tag):
    from infi.execute import execute_assert_success
    execute_assert_success("git checkout master", shell=True)
    execute_assert_success("git merge develop --no-ff -m \"Finished Release {}\"".format(version_tag), shell=True)
    execute_assert_success("git tag -a {0} -m {0}".format(version_tag), shell=True)
    execute_assert_success("git checkout develop", shell=True)
    execute_assert_success("git merge master", shell=True)

def git_checkout(branch_name_or_tag):
    from os import curdir
    from gitpy import LocalRepository
    logger.info("checking out '{}'".format(branch_name_or_tag))
    try:
        LocalRepository(curdir).checkout(branch_name_or_tag)
    except Exception:  # pragma: no cover
        logger.error("failed to checkout {}".format(branch_name_or_tag))
        raise SystemExit(1)

def commit_changes_to_buildout(message):
    import os
    from gitpy import LocalRepository
    repository = LocalRepository(os.curdir)
    # workaround https://github.com/msysgit/git/issues/79
    os.system("git status")
    if "buildout.cfg" not in [modified_file.filename for modified_file in repository.getChangedFiles()]:
        return
    repository.add("buildout.cfg")
    repository.commit("buildout.cfg: " + message)

def commit_changes_to_manifest_in(message):
    from os import curdir
    from gitpy import LocalRepository
    repository = LocalRepository(curdir)
    repository.add("MANIFEST.in")
    repository.commit("MANIFEST.in: " + message)

def get_latest_version():
    from os import curdir
    from gitpy import LocalRepository
    from pkg_resources import parse_version
    repository = LocalRepository(curdir)
    version_tags = [tag.name for tag in repository.getTags()
                    if tag.name.startswith('v') and not tag.name.endswith('-develop')]
    version_tags.sort(key=lambda ver: parse_version(ver))
    return version_tags[-1]

@contextmanager
def open_tempfile():
    from tempfile import mkstemp
    from os import close, remove
    fd, path = mkstemp()
    close(fd)
    try:
        yield path
    finally:
        try:
            remove(path)
        except:
            pass

def normalize(name):
    return re.sub(r"[-_.]+", "-", name).lower()

def set_freezed_versions_in_install_requires(buildout_cfg, versions_cfg):
    from .package_sets import InstallRequiresPackageSet, VersionSectionSet, from_dict, to_dict
    install_requires = to_dict(InstallRequiresPackageSet.from_value(buildout_cfg.get("project", "install_requires")))
    versions = to_dict(set([item.replace('==', ">=") for item in VersionSectionSet.from_value(versions_cfg)]))
    for key, value in versions.items():
        if key in install_requires and not install_requires[key]:  # empty list
            install_requires[key] = value
        else:
            for item in install_requires.items():
                if normalize(item[0]) == normalize(key) and not item[1]:
                    install_requires[item[0]] = value
                    break
    install_requires = from_dict(install_requires)
    buildout_cfg.set("project", "install_requires", InstallRequiresPackageSet.to_value(install_requires))

def freeze_versions(versions_file, change_install_requires):
    dependencies = dict()
    with open_buildout_configfile(write_on_exit=True) as buildout_cfg:
        with open_buildout_configfile(versions_file) as versions_cfg:
            if buildout_cfg.has_section("versions"):
                buildout_cfg.remove_section("versions")
            buildout_cfg.add_section("versions")
            for option in sorted(versions_cfg.options("versions"), key=lambda s: s.lower()):
                dependencies[option] = versions_cfg.get("versions", option)
        dependencies.update(**get_dependencies_with_specific_versions(buildout_cfg))
        for key in sorted(dependencies, key=lambda s: s.lower()):
            buildout_cfg.set("versions", key, dependencies[key])
        if change_install_requires:
                set_freezed_versions_in_install_requires(buildout_cfg, versions_cfg)

def unset_freezed_versions_in_install_requires(buildout_cfg):
    from .package_sets import InstallRequiresPackageSet, to_dict, from_dict
    install_requires = InstallRequiresPackageSet.from_value(buildout_cfg.get("project", "install_requires"))
    install_requires_dict = to_dict(install_requires)
    install_requires_dict.update({key: [] for key, specs in install_requires_dict.items()
                                  if specs and specs[-1][0] == '>='})
    install_requires = from_dict(install_requires_dict)
    buildout_cfg.set("project", "install_requires", InstallRequiresPackageSet.to_value(install_requires))


def get_dependencies_with_specific_versions(buildout_cfg):
    from .package_sets import InstallRequiresPackageSet, EggsPackageSet, to_dict
    results = {}
    install_requires = InstallRequiresPackageSet.from_value(buildout_cfg.get("project", "install_requires"))
    install_requires_dict = to_dict(install_requires)
    for key, specs in install_requires_dict.items():
        if specs and len(specs) == 1 and specs[0][0] == '==':
            results[key] = specs[0][1]

    development_eggs = [item for
                        item in EggsPackageSet.from_value(buildout_cfg.get('development-scripts', 'eggs')) if
                        item != '${project:name}']
    development_eggs_dict = to_dict(development_eggs)
    for key, specs in development_eggs_dict.items():
        if specs and len(specs) == 1 and specs[0][0] == '==':
            results[key] = specs[0][1]

    return results


def unfreeze_versions(change_install_requires):
    with open_buildout_configfile(write_on_exit=True) as buildout_cfg:
        buildout_cfg.remove_section("versions")

        dependencies_that_need_to_remain_frozen = get_dependencies_with_specific_versions(buildout_cfg)
        if dependencies_that_need_to_remain_frozen:
            buildout_cfg.add_section("versions")
            for key, value in dependencies_that_need_to_remain_frozen.items():
                buildout_cfg.set("versions", key, value)
        if change_install_requires:
            unset_freezed_versions_in_install_requires(buildout_cfg)


class RevertIfFailedOperations(object):
    def __init__(self, repository):
        super(RevertIfFailedOperations, self).__init__()
        self.repository = repository

    def get_tags(self):
        return {tag.name: tag for tag in self.repository.getTags()}

    def get_branches(self):
        return {branch.name: branch for branch in self.repository.getBranches()}

    def get_head(self, branch_name):
        return self.repository.getBranchByName(branch_name).getHead()

    def get_status(self):
        return dict(develop=self.get_head("develop"), master=self.get_head("master"),
                    tags=self.get_tags(), branches=self.get_branches())

    def delete_new_tags(self, before, now):
        for tag in set(now['tags']).difference(set(before['tags'])):
            cmd = "git tag -d {0}".format(now['tags'][tag].name)
            self.repository._executeGitCommandAssertSuccess(cmd)

    def delete_new_branches(self, before, now):
        for branch in set(now['branches']).difference(set(before['branches'])):
            cmd = "git branch -D {0}".format(now['branches'][branch].name)
            self.repository._executeGitCommandAssertSuccess(cmd)

    def reset_master_and_develop(self, before, now):
        for branch_name in ['master', 'develop']:
            self.repository.resetHard()
            branch = self.repository.getBranchByName(branch_name)
            self.repository.checkout(branch)
            self.repository.resetHard(before[branch_name])

@contextmanager
def revert_if_failed(keep_leftovers):
    from gitpy import LocalRepository
    from os import curdir
    repository = LocalRepository(curdir)
    ops = RevertIfFailedOperations(repository)
    before = ops.get_status()
    try:
        yield
    except:
        if keep_leftovers:
            raise
        repository.checkout('develop')
        now = ops.get_status()
        ops.delete_new_tags(before, now)
        ops.delete_new_branches(before, now)
        ops.reset_master_and_develop(before, now)
        raise

def release_version_in_git(version_tag, keep_leftovers=True):
    with revert_if_failed(keep_leftovers):
        _release_version_in_git(version_tag)
